/*
                Copyright (c) 1999 by
                Advanced Visual Systems Inc.
                All Rights Reserved

        This software comprises unpublished confidential information of
        Advanced Visual Systems Inc. and may not be used, copied or made
        available to anyone, except in accordance with the license
        under which it is furnished.

        This file is under Perforce control
        $Id: //depot/express/reduct/reduct/dsrLinks.h#2 $
*/

//============================================================================
// dsrLinks.h
// a class to maintain a list of links between the points of a triangular
// mesh and the triangles whose verices they are
//============================================================================

#ifndef __dsrLinks_h
#define __dsrLinks_h

#ifndef NULL
#define NULL    0L
#endif

struct dsrLink
{
  unsigned short ntris;
  int *tris;
};

class dsrLinks
{
public:
  dsrLink *links;

  dsrLinks() : links(NULL), maxId(0) {};
  dsrLinks(int nnodes) : maxId(nnodes) { this->links=new dsrLink[nnodes]; };

  void BuildLinks(int nnodes, int ntris, int *tris);
  void RemoveTriFromPoint(int triId, int p);
  void Resize(int p, int newSize);

  // Return number of tris used by pt
  unsigned short GetNTris(int ptId) { return this->links[ptId].ntris; };

  // Return a pts tri list
  int *GetTris(int ptId) { return this->links[ptId].tris; };

  // increase pts tri list by one (using Resize) and append new tri
  void AddTriToPoint(int triId, int p)
  {
    int oldntris;

    oldntris=this->links[p].ntris;
    this->Resize(p, oldntris+1);
    this->links[p].tris[oldntris]=triId;
  };

  // set ntris to 0 delete tri list and set to NULL
  void DeletePoint(int p)
  {
    this->links[p].ntris=0;
    delete [] this->links[p].tris;
    this->links[p].tris=NULL;
  };

  // remove tri from each of its pts
  void DeleteTri(int triId, int *tris)
  {
    int i, *pts;

    pts=&tris[triId*3];

    for(i=0; i<3; i++)
      // if pt hasn't already been deleted
      if(this->links[pts[i]].ntris)
        this->RemoveTriFromPoint(triId, pts[i]);
  };

  ~dsrLinks()
  {
    int i;

    if(!this->links)
      return;

    for(i=0; i<maxId; i++)
      delete [] this->links[i].tris;

    delete [] this->links;
  };

private:
  int maxId;
};

#endif
